#!/usr/bin/perl -w
# usage : modena.pl [options of MODENA] -f target_structure_filename
# 
# This script allows the user to specify "minimization" or "maximization" of 
# objective functions, if a "min" or "max" keyword is found in the righthand 
# side of the expression of an objective function. A space character 
# just before "min" or "max" is necessary. An example is shown below.
# 
# a:MFE  min
# ((b:CONT-50)^2)^0.5 min
#
# For the options of MODENA, please type "modena.pl -h" or "modena -h".
#
# In addition, modena.pl can perform the RNA sequence design with a 
# single objective function.
#
# ver.0.1 author: Akito Taneda

use strict ;

my $exe = 'modena ' ;

my $if = -1 ;
my $opt = '' ;
for (my $i = 0  ; $i <= $#ARGV ; $i++) {
   if ($ARGV[$i] eq '-f') { 
      $if = $i + 1 ;
      if ($i == $#ARGV) { die "ERROR: an input file is not found.\n" ; }
      $i++ ; # skip an input filename
    } else {
      $opt .= ' '.$ARGV[$i] ;
    }
}
if ($if == -1) { die "ERROR: option -f is not found.\n" ; }
my $inpf = $ARGV[$if] ;

unless (open(IN,$inpf)) {
   print "ERROR: input file cannot be opened.\n" ;
   exit (1);
} 
my @line = <IN> ;
chomp (@line) ;
close(IN) ;

my $tempf ;
my $wrkmax = 99999 ;
for (my $i = 1 ; $i <= $wrkmax ; $i++) {
   $tempf = 'modena.pl.temp.'.$i ;
   unless (-e $tempf) { last ; }
   if ($i == $wrkmax) { print "ERROR: a temp. filename cannot be creaetd.\n" ; exit (1) ; }
}

my $k ;
my @line2 ;
for ($k = 0  ; $k <= $#line; $k++) {
   push (@line2, $line[$k]) ;
   if ($line[$k] =~ /^;/) { last ; }
}

my $l ;
my $nof = 0 ;
for ($l = $k + 1 ; $l <= $#line; $l++) {
   if ($line[$l] =~ /^;/) { last ; }
   $nof++ ;
}

if ($nof == 1) { # when a sigle OF is specified.
   splice (@line, $k + 1, 0, $line[$k + 1]) ;
}

my @plusMinus ;
my $m = 0 ;
for ($l = $k + 1 ; $l <= $#line; $l++) {
   if ($line[$l] =~ /^;/) { last ; }
   elsif ($line[$l] =~ /\s+min\s*$/) {
      $line[$l] =~ s/\s+min\s*$// ;
      push (@line2, '-('.$line[$l].')') ;
      $plusMinus[$m++] = -1.0 ;

   } elsif ($line[$l] =~ /\s+max\s*$/) {
      $line[$l] =~ s/\s+max\s*$// ;
      push (@line2, $line[$l]) ;
      $plusMinus[$m++] = 1.0 ;
   } else {
      if ($line[$l] =~ /\s/) {
         $m++ ;
         die "ERROR: a max/min specification has an error OR please remove a space letter from the expression of OF $m\n" ;
      }

      push (@line2, $line[$l]) ;
      $plusMinus[$m++] = 1.0 ;
   }
}

for ($k = $l ; $k <= $#line ; $k++) {
   push (@line2, $line[$k]) ;
}

unless (open(TEMP,">$tempf")) {
   print "ERROR: temp. file cannot be opened.\n" ;
   exit (1);
} 
foreach (@line2) { print TEMP "$_\n" ; }
close (TEMP) ;

my @output = `$exe $opt -f $tempf` ;

foreach (@output) {
   if (/^Individual=/) {
      $_ =~ s/\n$// ;
      my @c = split (/Sc=\s*/, $_) ;
      print $c[0].'Sc=' ;
      my @d = split (/\s+/, $c[1]) ;
      if ($nof == 1) { # when a single OF is specified.
         printf " %f", $d[0]*$plusMinus[0] ;
      } else {
         for (my $x = 0 ; $x  <= $#d ; $x++) {
            printf " %f", $d[$x]*$plusMinus[$x] ;
         }
      }
      print "\n" ;
   } else {
      print ;
   }
}

system ("rm  $tempf") ;
