#!/usr/bin/perl -w
# A perl script for executing RNAstructure.
# single sequence only ! (the last sequence in a fasta file is used.)
#
# Usage:
# modena_dps.pl [fasta file] [Fold/EnsembleEnergy/efn2] ["options"] [constraint (optional for Fold)/bracket str. (mandatory for efn2)]
# Bracket notation: 
# Letters except ({[<>]})ABCDEabcde are treated as a loop position.
# author: Akito Taneda
use strict ;

my $maxout = 4 ; # #suboptimal structures to be outputted.
my $ss ;

my $prog = $ARGV[1] ;
my $options = $ARGV[2] ;
if ($prog ne 'efn2' and $prog ne 'Fold' and $prog ne 'EnsembleEnergy') {
   print "ERROR: a program name, $prog, is incorrect.\n" ;
   print "       (prog name =efn2, Fold, EnsembleEnergy)\n" ;
   exit (1);
}
if ($prog eq 'efn2') { $ss = $ARGV[3] ; }

my $inp = '' ; # input sequence
unless (open(IN,$ARGV[0])) {
   print "ERROR: input file cannot be opened.\n" ;
   exit (1);
} 
while (<IN>) {
   chomp;
   if (/^>\s*SecondaryStructure/) { last ; }
   if (/^>/) {
      $inp = ''
   } else {
      $_ =~ tr/a-z/A-Z/ ;
      $_ =~ s/[^ACGTU]/N/g ;
      $inp .= $_ ;
   }
}

my $len = length ($inp);
my $tempdir1 ; my $wrkmax = 99999 ;
for (my $i = 1 ; $i <= $wrkmax ; $i++) {
   $tempdir1 = 'modenaRNAstr'.$i ;
   unless (-e $tempdir1) { last ; }
   if ($i == $wrkmax) { print "ERROR: work dir. cannot be creaetd.\n" ; exit (1) ; }
}

my $root = `pwd` ;
$root =~ s/\n$// ;
mkdir ($tempdir1) ;
chdir ($tempdir1) ;

if ($prog eq 'Fold' and $#ARGV >= 3 and $ARGV[3] ne '') {
   my $sscon = $ARGV[3] ;
   my $confile = $ARGV[0] ;
   $confile =~ s/^.*\/// ;
   $confile = $confile.'.con' ;
   $options .= " -c $confile" ;

   my @bp ;
   @bp = &bra2ct ($sscon) ;
   open (CON, ">$confile") ;
   print CON "DS:\n-1\nSS:\n" ;
   # a loop constraint
   for (my $i = 1 ; $i <= $#bp ; $i++) {
      if ($bp[$i] == 0) { print CON "$i\n"; }
   }
   print CON "-1\nMod:\n-1\nPairs:\n" ;
   # a bp constraint
   for (my $i = 1 ; $i <= $#bp ; $i++) {
      if ($bp[$i] > 0 and $i <$bp[$i]) {
         print CON "$i $bp[$i]\n";
      }
   }
   print CON "-1 -1\nFMN:\n-1\nForbids:\n-1 -1\n" ;
   close (CON) ;
   #print  STDERR "CHKCON\n";
   #open(CHK,"$confile"); while(<CHK>) { print STDERR ; } close(CHK);
}

if ($prog eq 'efn2') {
   my @bp ;
   @bp = &bra2ct ($ss) ;
   &seqfile ($inp) ;
   &ctfile ($inp, @bp) ;
   `efn2 modena_dps.ct modena_dps.ene $options` ;
   open (ENE,"modena_dps.ene") ;
   my $ene = <ENE> ; # read an energy at line 1.
   close (ENE) ;
   $ene =~ s/^.*Energy =\s*// ;
   $ene =~ s/\s*\n$// ;
   print "$ene\n" ;

} elsif ($prog eq 'Fold') {
   &seqfile ($inp) ;
   `Fold modena_dps.seq modena_dps.ct $options` ; # For Mac, a fullpath should be specified, eg ~/RNAstructure/exe/Fold modena_dps.seq ... 
   &ct2bra ("modena_dps.ct", $inp, $maxout) ;

} elsif ($prog eq 'EnsembleEnergy') {
   &seqfile ($inp) ;
   my $ense = 0.0 ;
   `partition modena_dps.seq modena_dps.save $options` ;
   my @out = `EnsembleEnergy modena_dps.save` ;
   $ense = $out[3] ;
   $ense =~ s/^.*\:\s*// ;
   $ense =~ s/\s*kcal.*\n$// ;
   print "$ense\n" ;

} else {
   print "ERROR\n" ; exit (1) ;
}
chdir ($root) ;
system ("rm -rf $tempdir1") ;


sub seqfile {
   my ($inputSq) = @_ ;
   open (OUT,">modena_dps.seq");
   print OUT ";\n" ;
   print OUT "modena_dps_seq\n" ;
   print OUT "$inputSq" ;
   print OUT "1\n" ;
   close (OUT);
}


sub ctfile {
   my ($inputSq, @bx) = @_ ;
   my @sq = split (//, $inputSq) ;
   open (OUT,">modena_dps.ct") ;
   printf OUT "%5d  ENERGY = 0.0  modena_dps.seq\n", scalar (@bx) ;
   for (my $i = 1 ; $i <= $#bx - 1 ; $i++) {
      printf OUT "%5d %s   %5d%5d%5d%5d\n", $i, $sq[$i - 1], $i - 1, $i + 1, $bx[$i], $i ;
   }
   printf OUT "%5d %s   %5d%5d%5d%5d\n", $#bx, $sq[$#bx - 1], $#bx - 1, 0, $bx[$#bx], $#bx ;
   close (OUT) ;
   return @bx ;
}


sub bra2ct {
# input : $ss, output : @bp
   my ($ss) = @_ ;
   my (@bx, @st, $k) ;
   my ($bracket_type, %hash, @stack, @nsta) ;
   %hash = qw( .   0
               <   1
               >  -1
               \(  2
               \) -2
               {   3
               }  -3
               [   4
               ]  -4
               A   5
               a  -5
               B   6
               b  -6
               C   7
               c  -7
               D   8
               d  -8
               E   9
               e  -9 ) ;

   $bracket_type = ( scalar(keys (%hash)) - 1 )/2 ;
   for (my $i = 0 ; $i <= $bracket_type ; $i++) { # 0: a dummy
      $stack[$i] = () ;
      $nsta[$i] = 0 ;
   }
   @st = split(//, $ss) ;
   unshift (@st, 'Not used') ; # a dummy

   @bx = () ;
   for (my $i = 1 ; $i <= $#st ; $i++) {
      $bx[$i] = -1 ;
   }
   for (my $i = 1 ; $i <= $#st ; $i++) {
      if ( exists($hash{$st[$i]}) ) {
         if ($hash{ $st[$i] } > 0) {
            $k = $hash{ $st[$i] } ;
            $stack[$k][ $nsta[$k] ] = $i ;
            $nsta[$k]++ ;
         } elsif ($hash{ $st[$i] } < 0) {
            $k = -1*$hash{ $st[$i] } ;
            $nsta[$k]-- ;
            $bx[$i] = $stack[$k][ $nsta[$k] ] ;
            $bx[ $stack[$k][ $nsta[$k] ] ] = $i ;
         } else {
            $bx[$i] = 0 ;
         }
      }
   }

   # error check
   for (my $i = 1 ; $i <= $bracket_type ; $i++) {
      if ($nsta[$i] > 0) {
         die "ERROR: stack error. nsta[$i]=$nsta[$i]\n" ;
      }
   }
   return @bx ;
}


sub ct2bra {
# read .ct file, then output a structure to stdout in an RNAsubopt-like format.
   my ($ctFile, $inputSq, $max) = @_ ;
   open (CT, $ctFile) ;
   my @line = <CT> ;
   close (CT) ;
   my $m = 0 ;
   print "$inputSq 0 0\n" ; # for the format similar to that of RNAsubopt

   for (my $j = 0 ; $j < scalar (@line) ; ) {
      my @a = split (/\s+/, $line[$j]) ;
      shift(@a);
      my $n = $a[0] ;
      my $ene = $a[3] ;

      for (my $i = 1 ; $i <= $n ; $i++) {
         @a = split (/\s+/, $line[$i + $j]) ;
         shift (@a) ;
         if ($a[4] > 0 and $a[0] < $a[4]) { print "(" ; }
         elsif ($a[4] > 0 and $a[0] > $a[4]) { print ")" ; }
         else { print "." ; }
      }
      print " $ene\n" ;
      $j = $j + $n + 1 ;
      $m++ ;
   }
   if ($m < $max) {
      for (my $i = $m ; $i < $max ; $i++) {
         print '.' x length ($inputSq) ;
         print " 0.0\n" ;
      }
   }
}
